@@ -640,7 +640,7 @@ int dump_task_cgroup(struct pstree_item *item, u32 *cg_id, struct parasite_dump_
 	struct cg_set *cs;
 
 	if (item)
-		pid = item->pid.real;
+		pid = item->pid->real;
 	else
 		pid = getpid();
 
@@ -1510,7 +1510,7 @@ static int prepare_cgroup_dirs(char **controllers, int n_controllers, char *paux
 				return -1;
 			}
 
-			if (mkdirpat(cg, paux)) {
+			if (mkdirpat(cg, paux, 0755)) {
 				pr_perror("Can't make cgroup dir %s", paux);
 				return -1;
 			}
