@@ -3,7 +3,6 @@
 
 #include <sys/types.h>
 #include <string.h>
-#include "asm/string.h"
 
 #ifdef CONFIG_HAS_LIBBSD
 # include <bsd/string.h>
@@ -19,4 +18,6 @@ extern size_t strlcpy(char *dest, const char *src, size_t size);
 extern size_t strlcat(char *dest, const char *src, size_t count);
 #endif
 
+extern int builtin_strncmp(const char *cs, const char *ct, size_t count);
+
 #endif /* __CR_STRING_H__ */
