@@ -291,6 +291,8 @@ static void pr_info_ipc_shm(const IpcShmEntry *shm)
 	print_on_level(LOG_INFO, "size: %-10"PRIu64"\n", shm->size);
 }
 
+#define NR_MANDATORY_IPC_SYSCTLS 9
+
 static int ipc_sysctl_req(IpcVarEntry *e, int op)
 {
 	struct sysctl_req req[] = {
@@ -303,29 +305,37 @@ static int ipc_sysctl_req(IpcVarEntry *e, int op)
 		{ "kernel/shmall",		&e->shm_ctlall,		CTL_U64 },
 		{ "kernel/shmmni",		&e->shm_ctlmni,		CTL_U32 },
 		{ "kernel/shm_rmid_forced",	&e->shm_rmid_forced,	CTL_U32 },
-	};
-
-	struct sysctl_req req_mq[] = {
+		/* We have 9 mandatory sysctls above and 8 optional below */
 		{ "fs/mqueue/queues_max",	&e->mq_queues_max,	CTL_U32 },
 		{ "fs/mqueue/msg_max",		&e->mq_msg_max,		CTL_U32 },
 		{ "fs/mqueue/msgsize_max",	&e->mq_msgsize_max,	CTL_U32 },
 		{ "fs/mqueue/msg_default",	&e->mq_msg_default,	CTL_U32 },
 		{ "fs/mqueue/msgsize_default",	&e->mq_msgsize_default,	CTL_U32 },
+		{ "kernel/msg_next_id", 	&e->msg_next_id, 	CTL_U32 },
+		{ "kernel/sem_next_id", 	&e->sem_next_id, 	CTL_U32 },
+		{ "kernel/shm_next_id", 	&e->shm_next_id, 	CTL_U32 },
 	};
 
-	int ret;
+	int nr = NR_MANDATORY_IPC_SYSCTLS;
 
-	ret = sysctl_op(req, ARRAY_SIZE(req), op, CLONE_NEWIPC);
-	if (ret)
-		return ret;
-
-	if (access("/proc/sys/fs/mqueue", X_OK)) {
+	/* Skip sysctls which can't be set or haven't existed on dump */
+	if (access("/proc/sys/fs/mqueue", X_OK))
 		pr_info("Mqueue sysctls are missing\n");
-		return 0;
+	else {
+		nr += 3;
+		if (e->has_mq_msg_default) {
+			req[nr++] = req[12];
+			req[nr++] = req[13];
+		}
 	}
+	if (e->has_msg_next_id)
+		req[nr++] = req[14];
+	if (e->has_sem_next_id)
+		req[nr++] = req[15];
+	if (e->has_shm_next_id)
+		req[nr++] = req[16];
 
-	return sysctl_op(req_mq, ARRAY_SIZE(req_mq) - (e->has_mq_msg_default ? 0 : 2),
-	                 op, CLONE_NEWIPC);
+	return sysctl_op(req, nr, op, CLONE_NEWIPC);
 }
 
 /*
@@ -335,18 +345,36 @@ static int ipc_sysctl_req(IpcVarEntry *e, int op)
 static int dump_ipc_shm_pages(struct cr_img *img, const IpcShmEntry *shm)
 {
 	void *data;
-	int ret;
+	int ifd;
+	ssize_t size, off;
 
 	data = shmat(shm->desc->id, NULL, SHM_RDONLY);
 	if (data == (void *)-1) {
 		pr_perror("Failed to attach IPC shared memory");
 		return -errno;
 	}
-	ret = write_img_buf(img, data, round_up(shm->size, sizeof(u32)));
-	if (ret < 0) {
-		pr_err("Failed to write IPC shared memory data\n");
-		return ret;
-	}
+
+	/*
+	 * FIXME -- this just write the whole memory segment into the
+	 * image. In case the segment is huge this takes time. Need
+	 * to adopt the holes detection code (next_data_segment) from
+	 * shmem.c
+	 */
+	ifd = img_raw_fd(img);
+	size = round_up(shm->size, sizeof(u32));
+	off = 0;
+	do {
+		ssize_t ret;
+
+		ret = write(ifd, data + off, size - off);
+		if (ret <= 0) {
+			pr_perror("Failed to write IPC shared memory data");
+			return (int)ret;
+		}
+
+		off += ret;
+	} while (off < size);
+
 	if (shmdt(data)) {
 		pr_perror("Failed to detach IPC shared memory");
 		return -errno;
@@ -421,6 +449,9 @@ static int dump_ipc_var(struct cr_img *img)
 		goto err;
 	var.has_mq_msg_default = true;
 	var.has_mq_msgsize_default = true;
+	var.has_msg_next_id = true;
+	var.has_sem_next_id = true;
+	var.has_shm_next_id = true;
 
 	ret = ipc_sysctl_req(&var, CTL_READ);
 	if (ret < 0) {
@@ -428,6 +459,17 @@ static int dump_ipc_var(struct cr_img *img)
 		goto err;
 	}
 
+	/*
+	 * One can not write to msg_next_xxx sysctls -1,
+	 * which is their initial value
+	 */
+	if (var.msg_next_id == -1)
+		var.has_msg_next_id = false;
+	if (var.sem_next_id == -1)
+		var.has_sem_next_id = false;
+	if (var.shm_next_id == -1)
+		var.has_shm_next_id = false;
+
 	ret = pb_write_one(img, &var, PB_IPC_VAR);
 	if (ret < 0) {
 		pr_err("Failed to write IPC variables\n");
@@ -737,19 +779,30 @@ err:
 
 static int prepare_ipc_shm_pages(struct cr_img *img, const IpcShmEntry *shm)
 {
-	int ret;
 	void *data;
+	int ifd;
+	ssize_t size, off;
 
 	data = shmat(shm->desc->id, NULL, 0);
 	if (data == (void *)-1) {
 		pr_perror("Failed to attach IPC shared memory");
 		return -errno;
 	}
-	ret = read_img_buf(img, data, round_up(shm->size, sizeof(u32)));
-	if (ret < 0) {
-		pr_err("Failed to read IPC shared memory data\n");
-		return ret;
-	}
+
+	ifd = img_raw_fd(img);
+	size = round_up(shm->size, sizeof(u32));
+	off = 0;
+	do {
+		ssize_t ret;
+
+		ret = read(ifd, data + off, size - off);
+		if (ret <= 0) {
+			pr_perror("Failed to write IPC shared memory data");
+			return (int)ret;
+		}
+
+		off += ret;
+	} while (off < size);
 	if (shmdt(data)) {
 		pr_perror("Failed to detach IPC shared memory");
 		return -errno;
